@@ -3,6 +3,6 @@
 #ifndef BURN__ASYNC_H
 #define BURN__ASYNC_H
 
-void burn_async_join_all();
+void burn_async_join_all(void);
 struct burn_write_opts;
 #endif /* BURN__ASYNC_H */
