@@ -37,15 +37,21 @@ struct mempage;
 struct burn_drive *burn_drive_register(struct burn_drive *);
 
 unsigned int burn_drive_count(void);
-void burn_wait_all();
+void burn_wait_all(void);
 int burn_sector_length_write(struct burn_drive *d);
 int burn_track_control(struct burn_drive *d, int);
 void burn_write_empty_sector(int fd);
 void burn_write_empty_subcode(int fd);
-void burn_drive_free(void);
+void burn_drive_free(struct burn_drive *d);
+void burn_drive_free_all(void);
 
 int burn_drive_scan_sync(struct burn_drive_info *drives[],
 			 unsigned int *n_drives);
 void burn_disc_erase_sync(struct burn_drive *d, int fast);
+int burn_drive_get_block_types(struct burn_drive *d,
+			       enum burn_write_types write_type);
+
+/* ts A60822 */
+int burn_drive_is_open(struct burn_drive *d);
 
 #endif /* __DRIVE */
