@@ -3,6 +3,13 @@
 #ifndef LIBBURN_H
 #define LIBBURN_H
 
+/* Needed for  off_t  which is the (POSIX-ly) appropriate type for
+   expressing a file or stream size.
+
+   XXX we should enforce 64-bitness for off_t
+*/
+#include <sys/types.h>
+
 #ifndef DOXYGEN
 
 #if defined(__cplusplus)
@@ -73,6 +80,9 @@ struct burn_track;
 /** Track mode - audio
     2352 bytes per sector.  may be | with 4ch or preemphasis.
     NOT TO BE CONFUSED WITH BURN_MODE_RAW
+    Audio data must be 44100Hz 16bit stereo with no riff or other header at
+    beginning.  Extra header data will cause pops or clicks.  Audio data should
+    also be in little-endian byte order.  Big-endian audio data causes static.
 */
 #define BURN_AUDIO		(1 << 6)
 /** Track mode modifier - 4 channel audio. */
@@ -193,7 +203,7 @@ enum burn_message_warning
 /** Possible error messages */
 enum burn_message_error
 {
-	BURN_ERROR_FOO
+	BURN_ERROR_CANCELLED
 };
 
 /** Possible data source return values */
@@ -278,7 +288,7 @@ struct burn_source {
 	                                       int size);
 
 	/** Get the size of the source's data */
-	int (*get_size)(struct burn_source *);
+	off_t (*get_size)(struct burn_source *);
 
 	/** Clean up the source specific data */
 	void (*free_data)(struct burn_source *);
@@ -300,8 +310,14 @@ struct burn_drive_info
 	char vendor[9];
 	/** Name of the drive */
 	char product[17];
+	/** Revision of the drive */
+	char revision[5];
 	/** Location of the drive in the filesystem. */
 	char location[17];
+	/** This is currently the string which is used as persistent
+	    drive address. But be warned: there is NO GUARANTEE that this
+	    will stay so. Always use function  burn_drive_get_adr() to
+	    inquire a persisten address.       ^^^^^^ ALWAYS ^^^^^^ */
 
 	/** Can the drive read DVD-RAM discs */
 	unsigned int read_dvdram:1;
@@ -362,7 +378,7 @@ struct burn_drive_info
 	struct burn_drive *drive;
 };
 
-/** Messsages from the library */
+/** Messages from the library */
 struct burn_message
 {
 	/** The drive associated with the message. NULL if the error is not
@@ -406,14 +422,17 @@ struct burn_progress {
 	/** The number of sector */
 	int sectors;
 	/** The current sector being processed */
-	int current_sector;
+	int sector;
 };
 
 /** Initialize the library.
     This must be called before using any other functions in the library. It
     may be called more than once with no effect.
-    If is possible to 'restart' the library by shutting it down and
-    re-initializing it, though there is no good reason to do that.
+    It is possible to 'restart' the library by shutting it down and
+    re-initializing it. This is necessary if you follow the older and
+    more general way of accessing a drive via burn_drive_scan() and
+    burn_drive_grab(). See burn_drive_scan_and_grab() with its strong
+    urges and its explanations.
     @return Nonzero if the library was able to initialize; zero if
             initialization failed.
 */
@@ -433,6 +452,24 @@ void burn_finish(void);
 */
 void burn_set_verbosity(int level);
 
+/* ts A60813 */
+/** Set parameters for behavior on opening device files. To be called early
+    after burn_initialize() and before any bus scan. But not mandatory at all.
+    Parameter value 1 enables a feature, 0 disables.  
+    Default is (1,0,0). Have a good reason before you change it.
+    @param exclusive Try to open only devices which are not marked as busy
+                     and try to mark them busy if opened sucessfully. (O_EXCL)
+                     There are kernels which simply don't care about O_EXCL.
+                     Some have it off, some have it on, some are switchable.
+    @param blocking  Try to wait for drives which do not open immediately but
+                     also do not return an error as well. (O_NONBLOCK)
+                     This might stall indefinitely with /dev/hdX hard disks.
+    @param abort_on_busy  Unconditionally abort process when a non blocking
+                          exclusive opening attempt indicates a busy drive.
+                          Use this only after thorough tests with your app.
+*/
+void burn_preset_device_open(int exclusive, int blocking, int abort_on_busy);
+
 /** Returns a newly allocated burn_message structure. This message should be
     freed with burn_message_free() when you are finished with it.
     @return A message or NULL when there are no more messages to retrieve.
@@ -442,43 +479,127 @@ struct burn_message* burn_get_message(void);
 /** Frees a burn_message structure */
 void burn_message_free(struct burn_message *msg);
 
-/** Scans for drives. This function MUST be called until it returns nonzero.
+
+/* ts A60823 */
+/** Aquire a drive with known persistent address.This is the sysadmin friendly
+    way to open one drive and to leave all others untouched. It bundles
+    the following API calls to form a non-obtrusive way to use libburn:
+      burn_drive_add_whitelist() , burn_drive_scan() , burn_drive_grab()
+    You are *strongly urged* to use this call whenever you know the drive
+    address in advance.
+    If not, then you have to use directly above calls. In that case, you are
+    *strongly urged* to drop any unintended drive which will be exclusively
+    occupied and not closed by burn_drive_scan().
+    This can be done by shutting down the library including a call to
+    burn_finish(). You may later start a new libburn session and should then
+    use the function described here with an address obtained after
+    burn_drive_scan() via burn_drive_get_adr(&(drive_infos[driveno]), adr) .
+    Another way is to drop the unwanted drives by burn_drive_info_forget().
+    @param drive_infos On success returns a one element array with the drive
+                  (cdrom/burner). Thus use with driveno 0 only. On failure
+                  the array has no valid elements at all.
+                  The returned array should be freed via burn_drive_info_free()
+                  when it is no longer needed, and before calling a scan
+                  function again.
+                  This is a result from call burn_drive_scan(). See there.
+                  Use with driveno 0 only.
+    @param adr    The persistent address of the desired drive. Either obtained
+                  by burn_drive_get_adr() or guessed skillfully by application
+                  resp. its user.
+    @param load   Nonzero to make the drive attempt to load a disc (close its
+                  tray door, etc).
+    @return       1 = success , 0 = drive not found , -1 = other error
+*/    
+int burn_drive_scan_and_grab(struct burn_drive_info *drive_infos[],
+                             char* adr, int load);
+
+
+/* ts A51221 */
+/** Maximum number of particularly permissible drive addresses */
+#define BURN_DRIVE_WHITELIST_LEN 255
+/** Add a device to the list of permissible drives. As soon as some entry is in
+    the whitelist all non-listed drives are banned from scanning.
+    @return 1 success, <=0 failure
+*/
+int burn_drive_add_whitelist(char *device_address);
+
+/** Remove all drives from whitelist. This enables all possible drives. */
+void burn_drive_clear_whitelist(void);
+
+
+/** Scan for drives. This function MUST be called until it returns nonzero.
     No drives can be in use when this is called or it will assert.
     All drive pointers are invalidated by using this function. Do NOT store
     drive pointers across calls to this function or death AND pain will ensue.
-    When the app is done with the burn_drive_info array, it must be freed with
-    burn_drive_info_free()
-    @param drives Returns an array of drives (cdroms/burners). The returned
-                  array should be freed when it is no longer needed, and
-                  before calling this function again to rescan.
-    @param n_drives Returns the number of hardware drives in @c drives.
+    After this call all drives depicted by the returned array are subject
+    to eventual (O_EXCL) locking. See burn_preset_device_open(). This state
+    ends either with burn_drive_info_forget() or with burn_drive_release().
+    It is unfriendly to other processes on the system to hold drives locked
+    which one does not definitely plan to use soon.
+    @param drive_infos Returns an array of drive info items (cdroms/burners).
+                  The returned array must be freed by burn_drive_info_free()
+                  before burn_finish(), and also before calling this function
+                  burn_drive_scan() again.
+    @param n_drives Returns the number of drive items in drive_infos.
     @return Zero while scanning is not complete; non-zero when it is finished.
 */
-int burn_drive_scan(struct burn_drive_info *drives[],
-		       unsigned int *n_drives);
-/** Frees a burn_drive_info array returned by burn_drive_scan
-    @param info The array to free
+int burn_drive_scan(struct burn_drive_info *drive_infos[],
+		    unsigned int *n_drives);
+
+/* ts A60904 : ticket 62, contribution by elmom */
+/** Release memory about a single drive and any exclusive lock on it.
+    Become unable to inquire or grab it. Expect FATAL consequences if you try.
+    @param drive_info pointer to a single element out of the array
+                      obtained from burn_drive_scan() : &(drive_infos[driveno])
+    @param force controls degree of permissible drive usage at the moment this
+                 function is called, and the amount of automatically provided
+                 drive shutdown : 
+                  0= drive must be ungrabbed and BURN_DRIVE_IDLE
+                  1= try to release drive resp. accept BURN_DRIVE_GRABBING 
+                 Use these two only. Further values are to be defined.
+    @return 1 on success, 2 if drive was already forgotten,
+            0 if not permissible, <0 on other failures, 
 */
-void burn_drive_info_free(struct burn_drive_info *info);
+int burn_drive_info_forget(struct burn_drive_info *drive_info, int force);
+
+
+/** Free a burn_drive_info array returned by burn_drive_scan
+*/
+void burn_drive_info_free(struct burn_drive_info drive_infos[]);
+
+
+/* ts A60823 */
+/** Maximum length+1 to expect with a persistent drive address string */
+#define BURN_DRIVE_ADR_LEN 1024
+
+/** Inquire the persistent address of the given drive.
+    @param drive_info The drive to inquire. Usually some &(drive_infos[driveno])
+    @param adr   An application provided array of at least BURN_DRIVE_ADR_LEN
+                 characters size. The persistent address gets copied to it.
+*/
+int burn_drive_get_adr(struct burn_drive_info *drive_info, char adr[]);
+
 
 /** Grab a drive. This must be done before the drive can be used (for reading,
-    writing, etc). It may be neccesary to call this function more than once
-    to grab a drive. See burn_grab for details.
+    writing, etc).
     @param drive The drive to grab. This is found in a returned
                  burn_drive_info struct.
     @param load Nonzero to make the drive attempt to load a disc (close its
                 tray door, etc).
-    @return 1 if the drive has been grabbed, else 0
+    @return 1 if it was possible to grab the drive, else 0
 */
 int burn_drive_grab(struct burn_drive *drive, int load);
 
+
 /** Release a drive. This should not be done until the drive is no longer
-	busy (see burn_drive_get_status).
+    busy (see burn_drive_get_status). The drive is (O_EXCL) unlocked
+    afterwards.
 	@param drive The drive to release.
 	@param eject Nonzero to make the drive eject the disc in it.
 */
 void burn_drive_release(struct burn_drive *drive, int eject);
 
+
 /** Returns what kind of disc a drive is holding. This function may need to be
     called more than once to get a proper status from it. See burn_status
     for details.
@@ -497,7 +618,8 @@ int burn_disc_erasable(struct burn_drive *d);
     @param p Returns the progress of the operation, NULL if you don't care
     @return the current status of the drive. See also burn_drive_status.
 */
-enum burn_drive_status burn_drive_get_status(struct burn_drive *drive, struct burn_progress *p);
+enum burn_drive_status burn_drive_get_status(struct burn_drive *drive,
+					     struct burn_progress *p);
 
 /** Creates a write_opts struct for burning to the specified drive
     must be freed with burn_write_opts_free
@@ -615,7 +737,8 @@ void burn_session_free(struct burn_session *s);
 	@param pos position to add at (BURN_POS_END is "at the end")
 	@return 0 for failure, 1 for success
 */
-int burn_disc_add_session(struct burn_disc *d, struct burn_session *s, unsigned int pos);
+int burn_disc_add_session(struct burn_disc *d, struct burn_session *s,
+			  unsigned int pos);
 
 /** Remove a session from a disc
 	@param d Disc to remove session from
@@ -638,7 +761,8 @@ void burn_track_free(struct burn_track *t);
 	@param pos position to add at (BURN_POS_END is "at the end")
 	@return 0 for failure, 1 for success
 */
-int burn_session_add_track(struct burn_session *s, struct burn_track *t, unsigned int pos);
+int burn_session_add_track(struct burn_session *s, struct burn_track *t,
+			   unsigned int pos);
 
 /** Remove a track from a session
 	@param s Session to remove track from
@@ -658,7 +782,7 @@ int burn_session_remove_track(struct burn_session *s, struct burn_track *t);
 	@param mode data format (bitfield)
 */
 void burn_track_define_data(struct burn_track *t, int offset, int tail,
-                               int pad, int mode);
+			    int pad, int mode);
 
 /** Set the ISRC details for a track
 	@param t The track to change
@@ -670,7 +794,7 @@ void burn_track_define_data(struct burn_track *t, int offset, int tail,
 	@param serial 5 digit serial number. A number in 0-99999.
 */
 void burn_track_set_isrc(struct burn_track *t, char *country, char *owner,
-                            unsigned char year, unsigned int serial);
+			 unsigned char year, unsigned int serial);
 
 /** Disable ISRC parameters for a track
 	@param t The track to change
@@ -697,7 +821,7 @@ struct burn_disc *burn_drive_get_disc(struct burn_drive *d);
     
 */
 enum burn_source_status burn_track_set_source(struct burn_track *t,
-                                                    struct burn_source *s);
+					      struct burn_source *s);
 
 /** Free a burn_source (decrease its refcount and maybe free it)
 	@param s Source to free
@@ -706,7 +830,18 @@ void burn_source_free(struct burn_source *s);
 
 /** Creates a data source for an image file (and maybe subcode file) */
 struct burn_source *burn_file_source_new(const char *path,
-                                               const char *subpath);
+					 const char *subpath);
+
+/** Creates a data source for an image file (a track) from an open
+    readable filedescriptor, an eventually open readable subcodes file
+    descriptor and eventually a fixed size in bytes.
+    @param datafd The source of data.
+    @param subfd The eventual source for subcodes. Not used if -1.
+    @param size The eventual fixed size of eventually both fds. 
+                If this value is 0, the size will be determined from datafd.
+*/
+struct burn_source *burn_fd_source_new(int datafd, int subfd, off_t size);
+
 /** Tells how long a track will be on disc */
 int burn_track_get_sectors(struct burn_track *);
 
@@ -729,14 +864,18 @@ void burn_structure_print_track(struct burn_track *t);
     @param block_type The block type to use
     @return Returns 1 on success and 0 on failure.
 */
-int burn_write_opts_set_write_type(struct burn_write_opts *opts, enum burn_write_types write_type, int block_type);
+int burn_write_opts_set_write_type(struct burn_write_opts *opts,
+				   enum burn_write_types write_type,
+				   int block_type);
 
 /** Supplies toc entries for writing - not normally required for cd mastering
     @param opts The write opts to change
     @param count The number of entries
     @param toc_entries
 */
-void burn_write_opts_set_toc_entries(struct burn_write_opts *opts, int count, struct burn_toc_entry *toc_entries);
+void burn_write_opts_set_toc_entries(struct burn_write_opts *opts,
+				     int count,
+				     struct burn_toc_entry *toc_entries);
 
 /** Sets the session format for a disc
     @param opts The write opts to change
@@ -756,7 +895,8 @@ int  burn_write_opts_set_simulate(struct burn_write_opts *opts, int sim);
     @param underrun_proof if non-zero, buffer underrun protection is enabled
     @return Returns 1 on success and 0 on failure.
 */
-int burn_write_opts_set_underrun_proof(struct burn_write_opts *opts, int underrun_proof);
+int burn_write_opts_set_underrun_proof(struct burn_write_opts *opts,
+				       int underrun_proof);
 
 /** Sets whether to use opc or not with the write_opts struct
     @param opts The write opts to change
@@ -766,6 +906,10 @@ int burn_write_opts_set_underrun_proof(struct burn_write_opts *opts, int underru
 */
 void burn_write_opts_set_perform_opc(struct burn_write_opts *opts, int opc);
 
+void burn_write_opts_set_has_mediacatalog(struct burn_write_opts *opts, int has_mediacatalog);
+
+void burn_write_opts_set_mediacatalog(struct burn_write_opts *opts, unsigned char mediacatalog[13]);
+
 /** Sets whether to read in raw mode or not
     @param opts The read opts to change
     @param raw_mode If non-zero, reading will be done in raw mode, so that everything in the data tracks on the
@@ -783,37 +927,43 @@ void burn_read_opts_set_c2errors(struct burn_read_opts *opts, int c2errors);
     @param opts The read opts to change
     @param subcodes_audio If non-zero, read subcodes from audio tracks on the disc.
 */
-void burn_read_opts_read_subcodes_audio(struct burn_read_opts *opts, int subcodes_audio);
+void burn_read_opts_read_subcodes_audio(struct burn_read_opts *opts,
+					int subcodes_audio);
 
 /** Sets whether to read subcodes from data tracks or not 
     @param opts The read opts to change
     @param subcodes_data If non-zero, read subcodes from data tracks on the disc.
 */
-void burn_read_opts_read_subcodes_data(struct burn_read_opts *opts, int subcodes_data);
+void burn_read_opts_read_subcodes_data(struct burn_read_opts *opts,
+				       int subcodes_data);
 
 /** Sets whether to recover errors if possible
     @param opts The read opts to change
     @param hardware_error_recovery If non-zero, attempt to recover errors if possible.
 */
-void burn_read_opts_set_hardware_error_recovery(struct burn_read_opts *opts, int hardware_error_recovery);
+void burn_read_opts_set_hardware_error_recovery(struct burn_read_opts *opts,
+						int hardware_error_recovery);
 
 /** Sets whether to report recovered errors or not
     @param opts The read opts to change
     @param report_recovered_errors If non-zero, recovered errors will be reported.
 */
-void burn_read_opts_report_recovered_errors(struct burn_read_opts *opts, int report_recovered_errors);
+void burn_read_opts_report_recovered_errors(struct burn_read_opts *opts,
+					    int report_recovered_errors);
 
 /** Sets whether blocks with unrecoverable errors should be read or not
     @param opts The read opts to change
     @param transfer_damaged_blocks If non-zero, blocks with unrecoverable errors will still be read.
 */
-void burn_read_opts_transfer_damaged_blocks(struct burn_read_opts *opts, int transfer_damaged_blocks);
+void burn_read_opts_transfer_damaged_blocks(struct burn_read_opts *opts,
+					    int transfer_damaged_blocks);
 
 /** Sets the number of retries to attempt when trying to correct an error
     @param opts The read opts to change
     @param hardware_error_retries The number of retries to attempt when correcting an error.
 */
-void burn_read_opts_set_hardware_error_retries(struct burn_read_opts *opts, unsigned char hardware_error_retries);
+void burn_read_opts_set_hardware_error_retries(struct burn_read_opts *opts,
+					       unsigned char hardware_error_retries);
 
 /** Gets the maximum write speed for a drive
     @param d Drive to query
@@ -849,6 +999,8 @@ void burn_session_get_leadout_entry(struct burn_session *s,
 struct burn_session **burn_disc_get_sessions(struct burn_disc *d,
                                              int *num);
 
+int burn_disc_get_sectors(struct burn_disc *d);
+
 /** Gets an array of all the tracks for a session
     THIS IS NO LONGER VALID AFTER YOU ADD OR REMOVE A TRACK
     @param s session to get track array for
@@ -858,6 +1010,8 @@ struct burn_session **burn_disc_get_sessions(struct burn_disc *d,
 struct burn_track **burn_session_get_tracks(struct burn_session *s,
                                             int *num);
 
+int burn_session_get_sectors(struct burn_session *s);
+
 /** Gets the mode of a track
     @param track the track to query
     @return the track's mode
@@ -870,6 +1024,13 @@ int burn_track_get_mode(struct burn_track *track);
 */
 int burn_session_get_hidefirst(struct burn_session *session);
 
+/** Returns the library's version in its parts
+    @param major The major version number
+    @param minor The minor version number
+    @param micro The micro version number
+*/
+void burn_version(int *major, int *minor, int *micro);
+
 #ifndef DOXYGEN
 
 BURN_END_DECLS
