@@ -5,7 +5,7 @@
 
 #include "libburn.h"
 
-void burn_message_clear_queue();
+void burn_message_clear_queue(void);
 
 void burn_message_info_new(struct burn_drive *drive,
 			   enum burn_message_info message);
