@@ -16,7 +16,8 @@ void mmc_close_session(struct burn_drive *, struct burn_write_opts *);
 void mmc_close_disc(struct burn_drive *, struct burn_write_opts *);
 void mmc_close(struct burn_drive *, int session, int track);
 void mmc_get_event(struct burn_drive *);
-void mmc_write(struct burn_drive *, int start, struct buffer *buf);
+int mmc_write(struct burn_drive *, int start, struct buffer *buf);
+void mmc_write_12(struct burn_drive *d, int start, struct buffer *buf);
 void mmc_sync_cache(struct burn_drive *);
 void mmc_load(struct burn_drive *);
 void mmc_eject(struct burn_drive *);
