@@ -1,6 +1,8 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
+#include "null.h"
 #include "libburn.h"
+#include <stdlib.h>
 
 #include <string.h>
 int null_read(struct burn_source *source, unsigned char *buffer, int size)
