@@ -2,12 +2,6 @@
 
 /* scsi block commands */
 
-#include <unistd.h>
-#include <stdio.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <sys/ioctl.h>
 #include <scsi/scsi.h>
 #include <string.h>
 #include <scsi/sg.h>
