@@ -11,13 +11,13 @@ struct isrc;
 
 int dec_to_bcd(int);
 
-void sector_toc(struct burn_write_opts *, int mode);
-void sector_pregap(struct burn_write_opts *, unsigned char tno,
+int sector_toc(struct burn_write_opts *, int mode);
+int sector_pregap(struct burn_write_opts *, unsigned char tno,
 		   unsigned char control, int mode);
-void sector_postgap(struct burn_write_opts *, unsigned char tno,
+int sector_postgap(struct burn_write_opts *, unsigned char tno,
 		    unsigned char control, int mode);
-void sector_lout(struct burn_write_opts *, unsigned char control, int mode);
-void sector_data(struct burn_write_opts *, struct burn_track *t, int psub);
+int sector_lout(struct burn_write_opts *, unsigned char control, int mode);
+int sector_data(struct burn_write_opts *, struct burn_track *t, int psub);
 void sector_headers(struct burn_write_opts *, unsigned char *,
 		    int mode, int leadin);
 void subcode_user(struct burn_write_opts *, unsigned char *s,
