@@ -19,12 +19,18 @@ struct burn_track
 	unsigned int index[99];
 	/** number of 0 bytes to write before data */
 	int offset;
+	/** how much offset has been used */
+	int offsetcount;
 	/** Number of zeros to write after data */
 	int tail;
+	/** how much tail has been used */
+	int tailcount;
 	/** 1 means Pad with zeros, 0 means start reading the next track */
 	int pad;
 	/** Data source */
 	struct burn_source *source;
+	/** End of Source flag */
+	int eos;
 	/** The audio/data mode for the entry. Derived from control and
 	    possibly from reading the track's first sector. */
 	int mode;
