@@ -1,6 +1,9 @@
 #include <string.h>
 #include <assert.h>
 #include <stdlib.h>
+#include "../version.h"
+#include "util.h"
+#include "libburn.h"
 
 char *burn_strdup(char *s)
 {
@@ -32,3 +35,10 @@ char *burn_strndup(char *s, int n)
 
 	return ret;
 }
+
+void burn_version(int *major, int *minor, int *micro)
+{
+    *major = BURN_MAJOR_VERSION;
+    *minor = BURN_MINOR_VERSION;
+    *micro = BURN_MICRO_VERSION;
+}
